package com.gic.auth.service.outer.impl;

import java.util.Date;
import java.util.List;

import com.gic.auth.entity.TabSysResource;
import com.gic.auth.entity.TabSysRole;
import com.gic.auth.utils.ValidSplitUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserListDTO;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.dto.UserRoleDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.store.utils.valid.ValidUtil;

/**
 * @author guojx
 * @date 2019/7/16 6:55 PM
 */
@Service("userApiService")
public class UserApiServiceImpl implements UserApiService {
    @Autowired
    private UserService userService;
    @Autowired
    private UserRoleApiService userRoleApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private ResourceService resourceService;
    @Autowired
    private RoleService roleService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> saveUser(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.SaveUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (userService.isPhoneRepeat(null, userDTO.getPhoneNumber(), userDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }

        //如果是普通管理员，则需要验证角色权限和资源权限
        if (userDTO.getSuperAdmin().intValue() == 0) {
            if (StringUtils.isBlank(userDTO.getUserRoleIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色权限不能为空");
            }
            if (StringUtils.isBlank(userDTO.getUserResourceIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源权限不能为空");
            }
        }

        if (userDTO.getSuperAdmin() == null) {
            //默认是普通管理员
            userDTO.setSuperAdmin(0);
        }
        if (userDTO.getSuperAdmin().intValue() == 1) {
            //超级官员
            TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
            if (adminUser != null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已存在超级管理员，重复创建");
            }
        }

        if (userDTO.getPasswordType() == null) {
            //默认自定义创建密码类型
            userDTO.setPasswordType(2);
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode("86");
        }
        userDTO.setCreateTime(new Date());
        userDTO.setUpdateTime(new Date());
        userDTO.setStatus(1);
        Integer userId = userService.saveUser(userDTO);

        userRoleService.deleteByUserId(userId);
        userResourceService.deleteByUserId(userId);

        ServiceResponse<Integer> response = saveRole(userId, userDTO.getEnterpriseId(), userDTO.getUserRoleIds(),
                userDTO.getUserResourceIds());
        if (response.isSuccess()) {
            return response;
        } else {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return response;
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse editUser(UserDTO userDTO) {
        //valid param
        ServiceResponse paramResult = ValidUtil.allCheckValidate(userDTO, UserDTO.EditUserValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        TabSysUser tabUser = userService.getUserById(userDTO.getUserId());
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误");
        }
        if (userService.isPhoneRepeat(userDTO.getUserId(), userDTO.getPhoneNumber(), tabUser.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "手机号码不能重复");
        }

        if (userDTO.getSuperAdmin() != null && userDTO.getSuperAdmin().intValue() == 1) {
            //超级官员
            TabSysUser adminUser = userService.getUserByEnterpriseId(userDTO.getEnterpriseId());
            if (adminUser != null && adminUser.getUserId().intValue() != tabUser.getUserId().intValue()) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已存在超级管理员，参数有误");
            }
        }

        if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
            userDTO.setPhoneAreaCode("86");
        }
        userDTO.setUpdateTime(new Date());
        userService.editUser(userDTO);

        userRoleService.deleteByUserId(userDTO.getUserId());
        userResourceService.deleteByUserId(userDTO.getUserId());

        ServiceResponse response = saveRole(userDTO.getUserId(), userDTO.getEnterpriseId(), userDTO.getUserRoleIds(),
                userDTO.getUserResourceIds());
        if (response.isSuccess()) {
            return response;
        } else {
            //事物回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return response;
        }
    }

    @Override
    public ServiceResponse<UserDTO> getUserById(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误");
        }
        UserDTO result = EntityUtil.changeEntityNew(UserDTO.class, tabUser);
        //角色集
        List<UserRoleDTO> userRoleDTOList = userRoleService.listUserRoleByUserId(userId);
        StringBuilder userRoleStr = new StringBuilder();
        for (UserRoleDTO userRoleDTO : userRoleDTOList) {
            userRoleStr.append(userRoleDTO.getRoleId()).append(",");
        }

        String userRoleResult = userRoleStr.toString();
        result.setUserRoleIds(userRoleResult.substring(0, userRoleResult.length() - 1));
        //资源集
        List<UserResourceDTO> userResourceDTOList = userResourceService.listUserResourceByUserId(userId);
        StringBuilder userResourceStr = new StringBuilder();
        for (UserResourceDTO userResourceDTO : userResourceDTOList) {
            userResourceStr.append(userResourceDTO.getResourceId()).append(",");
        }
        String userResourceResult = userResourceStr.toString();
        result.setUserResourceIds(userResourceResult.substring(0, userResourceResult.length() - 1));
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Void> delete(Integer userId) {
        TabSysUser tabUser = userService.getUserById(userId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户ID输入有误，无此数据");
        }
        //删除角色关联
        userRoleService.deleteByUserId(userId);
        //删除资源关联
        userResourceService.deleteByUserId(userId);
        userService.delete(userId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> getUserByEnterpriseId(Integer enterpriseId) {
        TabSysUser tabUser = userService.getUserByEnterpriseId(enterpriseId);
        if (tabUser == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID输入有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(UserDTO.class, tabUser));
    }

    @Override
    public ServiceResponse<List<UserDTO>> listUserByPhoneNumber(String phoneNumber) {
        List<TabSysUser> list = this.userService.listAllUserByPhoneNumber(phoneNumber);
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListByJSON(UserDTO.class, list));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserDTO> login(String phoneNumber, Integer enterpriseId, String password) {
        TabSysUser user = this.userService.login(phoneNumber, enterpriseId, password);
        return ServiceResponse.success(EntityUtil.changeEntityByJSON(UserDTO.class, user));
    }

    @Override
    public ServiceResponse<Page<UserListDTO>> pageUser(UserListQO params) {
        com.github.pagehelper.Page page = userService.pageUser(params);
        Page<UserListDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    /**
     * 保存关联数据，角色关联、资源关联
     * @Title: saveRole 
     * @Description:
      * @author guojuxing
     * @param userId
     * @param enterpriseId
     * @param userRoleIds
     * @param userResourceIds 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    private ServiceResponse<Integer> saveRole(Integer userId, Integer enterpriseId, String userRoleIds,
            String userResourceIds) {
        String[] userRoleIdArr;
        String[] userResourceIdArr;
        ServiceResponse<String[]> userRoleValid = ValidSplitUtils.validStr(userRoleIds);
        if (userRoleValid.isSuccess()) {
            userRoleIdArr = userRoleValid.getResult();
        } else {
            return ServiceResponse.failure(userRoleValid.getCode(), userRoleValid.getMessage());
        }
        ServiceResponse<String[]> userResourceIdValid = ValidSplitUtils.validStr(userResourceIds);
        if (userResourceIdValid.isSuccess()) {
            userResourceIdArr = userResourceIdValid.getResult();
        } else {
            return ServiceResponse.failure(userResourceIdValid.getCode(), userResourceIdValid.getMessage());
        }
        //新增用户角色关联数据
        UserRoleDTO userRoleDTO = new UserRoleDTO();
        userRoleDTO.setEnterpriseId(enterpriseId);
        userRoleDTO.setUserId(userId);
        for (String userRole : userRoleIdArr) {
            int roleId = Integer.parseInt(userRole);
            TabSysRole tabSysRole = roleService.getById(roleId);
            if (tabSysRole == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "角色数据有误，无此角色数据");
            }
            userRoleDTO.setRoleId(roleId);
            ServiceResponse result = userRoleApiService.save(userRoleDTO);
            if (!result.isSuccess()) {
                return result;
            }
        }
        //新增用户资源关联数据
        UserResourceDTO userResourceDTO = new UserResourceDTO();
        userResourceDTO.setUserId(userId);
        userResourceDTO.setEnterpriseId(enterpriseId);
        for (String userResource : userResourceIdArr) {
            int resourceId = Integer.parseInt(userResource);
            TabSysResource tabSysResource = resourceService.getResource(resourceId);
            if (tabSysResource == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "资源数据有误，无此资源数据");
            }
            userResourceDTO.setResourceId(resourceId);
            ServiceResponse result = userResourceApiService.save(userResourceDTO);
            if (!result.isSuccess()) {
                return result;
            }
        }
        return ServiceResponse.success(userId);
    }

}
